/* FILE: showdriver.c             Copyright (c), Symmetric Research, 2007

This utility program shows the currently installed PARxCH drivers.

*/


#include <windows.h>
#include <stdio.h>

#include "parxchkd.h" // for device name registry parameter

// Defines and globals

#define NOT_FATAL   0
#define FATAL       1
#define MAXSTR      256


char *DriverOptions[] = {
        "SrParXch0",
        "SrParXch1",
        "SrParXch2",
        "SrParXch378",
        "SrParXch278",
        NULL
        };
#define OLDSTYLE 3 // DriverOptions[OLDSTYLE] is first old style name checked

char DriverName[MAXSTR], msg[MAXSTR];


char *UsageMsg[] = {
  "USAGE: showdriver                                                      ",
  "                                                                       ",
  "This utility shows the parameters for all installed PARxCH drivers.    ",
  "These parameters include the parallel port address and which Xch Model ",
  "will be used by default when running application programs such as diag,",
  "dvm and scope.                                                         ",
  "                                                                       ",
  "EXAMPLE: showdriver                                                    ",
  "                                                                       ",
   NULL
   };

//Protos

void Usage( char *msg );
void PrintMsg( char *MsgArray[] );
void Error( char *msg, int IsFatal );


// Main program

void main( int argc, char** argv ) {

        int   i;
        char  ServiceParmKeyName[MAXSTR], CurrentModel[MAXSTR];
        DWORD CurrentAddress, CurrentIrq;
        HKEY  ServiceParmKey;
        DWORD Code, BufferSize, RegType;

        // Print banner

        printf( "PARxCH ShowDriver : (c) SR 2007 : Rev {01/08/07}\n" );


        // Check for usage message request
        
        if ( argc > 1  ) {
                if ( strcmp( *argv, "/?" ) == 0 )
                        Usage( "?" );
                else
                        Usage( "Unknown argument" );
                }



        printf( "\nCurrent Driver Status:\n\n" );

        i = 0;
        while ( DriverOptions[i] ) {
                
                strcpy( DriverName, DriverOptions[i] );
                CurrentModel[0] = '\0';
                CurrentAddress  = 0;
                CurrentIrq      = 0;


                // Open service parameters registry key

                sprintf( ServiceParmKeyName,
                         "System\\CurrentControlSet\\Services\\%s\\Parameters",
                         DriverName );

                Code = RegOpenKeyEx(
                               HKEY_LOCAL_MACHINE,      // already open key
                               ServiceParmKeyName,      // subkey to open
                               0,                       // reserved
                               KEY_ALL_ACCESS,          // full access permission
                               &ServiceParmKey          // handle to key
                              );


                // If key was opened, driver is installed so get parameters
                
                if (Code == ERROR_SUCCESS) {

                        BufferSize = sizeof( CurrentModel );
                        Code = RegQueryValueEx(
                                               ServiceParmKey,
                                               REGENVSTR_MODELNAME,
                                               NULL,
                                               &RegType,
                                               CurrentModel,
                                               &BufferSize
                                              );
                        if (Code != ERROR_SUCCESS)
                                printf( "  Error: Could not determine XchModel (err = %d).\n", Code );



                        BufferSize = sizeof( CurrentAddress );
                        Code = RegQueryValueEx(
                                               ServiceParmKey,
                                               REGENVSTR_PORTADDR,
                                               NULL,
                                               &RegType,
                                               (LPBYTE)&CurrentAddress,
                                               &BufferSize
                                              );
                        if (Code != ERROR_SUCCESS)
                                printf( "  Error: Could not determine address (err = %d).\n", Code );


                        BufferSize = sizeof( CurrentIrq );
                        Code = RegQueryValueEx(
                                               ServiceParmKey,
                                               REGENVSTR_PORTIRQ,
                                               NULL,
                                               &RegType,
                                               (LPBYTE)&CurrentIrq,
                                               &BufferSize
                                              );
                        if (Code != ERROR_SUCCESS)
                                printf( "  Error: Could not determine irq (err = %d).\n", Code );

                        

                        // Announce current PARxCH driver registry parameters

                        printf( "  Driver %s is  installed with:\n", DriverName );
                        printf( "       XchModel = %s\n",   CurrentModel );
                        printf( "       Address  = 0x%X\n", CurrentAddress );
                        printf( "       IRQ      = %d\n\n", CurrentIrq );



                        Code = RegCloseKey( ServiceParmKey );
                        if (Code != ERROR_SUCCESS)
                                printf( "  Error: Could not close registry key (err = %d).\n", Code );

                        }


                // Else key does not open and driver is not installed
                // Only display "not installed" notice for new style drivers
                
                else
                        if ( i < OLDSTYLE )
                                printf( "  Driver %s not installed.\n", DriverName );
                

                
                // Move to next driver
                
                i++;
                
                } // end while
}

void Usage( char *msg ) {

        // Print a command line usage message and quit.

        // Select error line depending on 
        // 1) no message, 2) help requested, or 3) message given

        if ( !msg )
                printf( "\nERROR: Command line usage.\n\n" );

        else if ( strcmp( msg, "?" ) == 0 )
                printf( "\n" );

        else
                printf( "\nERROR: Command line usage (%s).\n\n", msg );

        PrintMsg( UsageMsg );
        exit( 1 );
}

void PrintMsg( char *MsgArray[] ) {

        int i;
        
        for ( i = 0 ; MsgArray[i] ; i++ )
                printf( "%s\n", MsgArray[i] );
}

void Error( char *msg, int IsFatal ) {
        
        printf( "ERROR: %s\n", msg );
        if ( IsFatal )
                exit( 0 );
}
