Option Strict On
Option Explicit On
Imports SIO = System.IO

Public Class Acquire
    Inherits System.Windows.Forms.Form

    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    ' ROUTINE: Acquire_Load                                                  '
    ' PURPOSE: Initialize the demo application and load its form.            '
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    Private Sub Acquire_Load(ByVal eventSender As System.Object, _
                          ByVal eventArgs As System.EventArgs) _
                          Handles MyBase.Load



       ' If the PARxCH library is compiled with GPS_AVAILABLE, the
       ' PARxCH DLL contains a call to ParGpsGetKernelArea which is
       ' located in the PARGPS DLL.  But VB does not appear to
       ' recognize this fact.  So we put a dummy call here to force
       ' VB into loading the function.  If this is not included, VB
       ' claims it can't find the PARxCH DLL.  Of course, your
       ' project must also include the pargps/vbasic/pargps.bas
       ' module file so this function is defined.

        ' NOTE: A call to the PARGPS library *** MUST *** come before
        '       any call to the PARxCH library.  This is because the
        '       PARxCH library calls a function in the PARGPS library
        '       but Visual Basic does not know how to process this
        '       nesting.  So, you must make a PARGPS library call first
        '       so VB pulls in the PARGPS library.  Then it will be
        '       able to successfully read in the PARxCH library since
        '       it already knows where the PARGPS library is.
       
#If GPS_AVAILABLE Then
       Dim Dummy As Integer
       Dummy = ParGpsGetKernelArea(PARGPS_BAD_DEVHANDLE, Dummy)
#End If


        ' One way of setting the working directory to path of the exe
        ' (does not work right for UNC paths given from Dos Prompt)
        '
        ' On Error Resume Next
        ' ChDrive(My.Application.Info.DirectoryPath)
        ' ChDir(My.Application.Info.DirectoryPath)
        ' WorkingDir = My.Application.Info.DirectoryPath
        '
        ' But with Visual Studio 2005, we don't want to use the exe path.
        ' Instead, we want to use the directory specified by the user as
        ' the "Start Options Working Directory" in the project debug tab
        ' or as the "Start in" field in the shortcut tab of the properties
        ' dialog for a shortcut.  Setting either field will cause the
        ' program to start there, so we just get the current directory.


        ' Initialize some variables

        WorkingDir = SIO.Directory.GetCurrentDirectory
        Call ParXchInitErrorMsg()


        ' Initialize driver name list

        DriverNameLst.Items.Clear()

        DriverNameLst.Items.Add( "SrParXch0" ) ' item 0
        DriverNameLst.Items.Add( "SrParXch1" ) ' item 1
        DriverNameLst.Items.Add( "SrParXch2" ) ' item 2

        DriverNameLst.SetSelected(0, True)



        ' Initialize Xch model name list

        ModelNameLst.Items.Clear()
        
        ModelNameLst.Items.Add("DEFAULT")        ' item 0
        ModelNameLst.Items.Add(PAR1CH_MODELNAME) ' item 1
        ModelNameLst.Items.Add(PAR4CH_MODELNAME) ' item 2
        ModelNameLst.Items.Add(PAR8CH_MODELNAME) ' item 3
        
        ModelNameLst.SetSelected(0, True)



        ' Set initial state
        
        DemoState = DEMO_STOPPED
        StopBtn.Enabled = False
        FileNumLbl.Enabled = False
        FileNumTxt.Enabled = False

     End Sub

     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
     ' ROUTINE: RunBtn_Click                                                  '
     ' PURPOSE: Start the demo running.  It acquires and writes out data.     '
     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
     Private Sub RunBtn_Click(ByVal eventSender As System.Object, _
                              ByVal eventArgs As System.EventArgs) _
                              Handles RunBtn.Click

        Dim Nread As Integer
        Dim ADS1210_CrValue As Integer
        Dim SrError As Integer
        Dim NextPt As Integer
        Dim Nremain As Integer
        Dim DataFormat As Integer
        Dim PointsPerChan As Integer
        Dim SampleRate As Double
        Dim GainLog As Integer
        Dim BufferFull As Short
        Dim ModelIndex As Integer


        ' Get driver, model and port information from controls

        DriverName = DriverNameLst.SelectedItem.ToString()
        
        ModelIndex = ModelNameLst.SelectedIndex
        If (ModelIndex = 1)
            XchModel = PARXCH_MODEL_PAR1CH
        Else If (ModelIndex = 2)
            XchModel = PARXCH_MODEL_PAR4CH
        Else If (ModelIndex = 3)
            XchModel = PARXCH_MODEL_PAR8CH
        Else
           XchModel = ParXchGetDefaultModel(DriverName)
        End If

        If (XchModel = PARXCH_MODEL_PAR1CH) Then
           ParXchAnalogChannels = PAR1CH_ANALOG_CHANNELS
           ModelName = PAR1CH_MODELNAME
        ElseIf (XchModel = PARXCH_MODEL_PAR4CH) Then
           ParXchAnalogChannels = PAR4CH_ANALOG_CHANNELS
           ModelName = PAR4CH_MODELNAME
        ElseIf (XchModel = PARXCH_MODEL_PAR8CH) Then
           ParXchAnalogChannels = PAR8CH_ANALOG_CHANNELS
           ModelName = PAR8CH_MODELNAME
        Else
           MsgWords = "Unknown Xch Model # " & Str(XchModel) & vbCrLf
           MsgResponse = MsgBox(MsgWords, MsgBoxStyle.OKOnly, "ERR_DEVID")
           Me.Close()
           Exit Sub
        End If



        If (PortModeOptBpp.Checked = True) Then
           PortMode = PARXCH_PORT_MODE_BPP
        ElseIf (PortModeOptEpp.Checked = True) Then
           PortMode = PARXCH_PORT_MODE_EPP
        ElseIf (PortModeOptEcpBpp.Checked = True) Then
           PortMode = PARXCH_PORT_MODE_ECP_BPP
        ElseIf (PortModeOptEcpEpp.Checked = True) Then
           PortMode = PARXCH_PORT_MODE_ECP_EPP
        Else
           PortMode = PARXCH_PORT_MODE_ECP_BPP
        End If



        ' Turn driver/model/port controls off and fifo/file controls on

        DriverNameLbl.Enabled = False
        DriverNameLst.Enabled = False
        ModelNameLbl.Enabled = False
        ModelNameLst.Enabled = False
        PortModeFra.Enabled = False
        PortModeOptBpp.Enabled = False
        PortModeOptEpp.Enabled = False
        PortModeOptEcpBpp.Enabled = False
        PortModeOptEcpEpp.Enabled = False

        FileNumLbl.Enabled = True
        FileNumTxt.Enabled = True

        FileNumTxt.Text = "None"
        FileNumTxt.Refresh()

        FileNum = 0


        ' Open and initialize PARxCH

        DataFormat = PARXCH_DF_SIGNED
        GainLog = PARXCH_GAIN_1
        SampleRate = 100# ' to match PARXCH_DECIMATION_100Hz_T16

        ParXchHandle = ParXchFullOpen(DriverName, _
                  XchModel, _
                  PortMode, _
                  DataFormat, _
                  GainLog, _
                  PARXCH_TURBO_16, _
                  PARXCH_DECIMATION_100Hz_T16, _
                  PARXCH_EXTRADECIMATION_MIN, _
                  PARXCH_UNUSED, _
                  ADS1210_CrValue, _
                  SrError)

        If (ParXchHandle = PARXCH_BAD_DEVHANDLE) Then
           MsgWords = "Could not initialize " & ModelName & " board" & vbCrLf
           MsgWords = MsgWords & "Error is " & Str(SrError) & vbCrLf
           MsgWords = MsgWords & PARXCH_ERROR_MSG(SrError)
           MsgResponse = MsgBox(MsgWords, MsgBoxStyle.OKOnly, "ERR_INIT")
           Me.Close()
           Exit Sub
        End If



        ' Dimension data array to hold buffer size

        PointsPerChan = 128 ' This is an arbitrary, but reasonable number
        Nvalues = PointsPerChan * ParXchAnalogChannels

        ReDim DataArray(Nvalues - 1)



        ' Write out header file

        Call Write_Header(ParXchAnalogChannels, _
                  PointsPerChan, _
                  DataFormat, _
                  SampleRate, _
                  GainLog)



        ' Begin acquisition

        StopBtn.Enabled = True
        RunBtn.Enabled = False
        DemoState = DEMO_RUNNING
        
        Call ParXchStart(ParXchHandle)

        Do While (DemoState = DEMO_RUNNING)

           BufferFull = 0
           NextPt = 0
           Nremain = Nvalues

           Do While (BufferFull = 0)

              ' Read any ready data

              Nread = ParXchReadData(ParXchHandle, DataArray(NextPt), Nremain, SrError)

              Nremain = Nremain - Nread
              NextPt = NextPt + Nread


              ' Check for a fatal error

              If (SrError <> PARXCH_ERROR_NONE) Then
                 MsgWords = "Read Data Error: "
                 MsgWords = MsgWords & PARXCH_ERROR_MSG(SrError)
                 MsgResponse = MsgBox(MsgWords, MsgBoxStyle.OKOnly, "ERR_READ")
                 Call ExitBtn_Click(ExitBtn, New System.EventArgs())
                 Exit Sub
              End If


              ' Check for buffer full

              If (Nremain <= 0) Then
                 BufferFull = 1
              End If


              ' Check to see if stop button has been pressed
              System.Windows.Forms.Application.DoEvents()
              If (DemoState <> DEMO_RUNNING) Then
                 BufferFull = 1
              End If

           Loop  ' on !BufferFull


           ' Write data to a file and disply file number

           Call Write_Data()
           FileNumTxt.Text = Str(FileNum - 1)
           FileNumTxt.Refresh()

           ' Check to see if stop button has been pressed
           System.Windows.Forms.Application.DoEvents()

        Loop  ' on DEMO_RUNNING

     End Sub

     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
     ' ROUTINE: StopBtn_Click                                                 '
     ' PURPOSE: Stop the demo from running.                                   '
     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
     Private Sub StopBtn_Click(ByVal eventSender As System.Object, _
                               ByVal eventArgs As System.EventArgs) _
                               Handles StopBtn.Click

        ' Stop and close PARxCH

        Call ParXchStop(ParXchHandle)
        DemoState = DEMO_STOPPED
        RunBtn.Enabled = True
        StopBtn.Enabled = False

        Call ParXchClose(ParXchHandle)


        ' Turn driver/model/port controls on and fifo/file controls off

        DriverNameLbl.Enabled = True
        DriverNameLst.Enabled = True
        ModelNameLbl.Enabled = True
        ModelNameLst.Enabled = True
        PortModeFra.Enabled = True
        PortModeOptBpp.Enabled = True
        PortModeOptEpp.Enabled = True
        PortModeOptEcpBpp.Enabled = True
        PortModeOptEcpEpp.Enabled = True

        FileNumLbl.Enabled = False
        FileNumTxt.Enabled = False

     End Sub

     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
     ' ROUTINE: ExitBtn_Click                                                 '
     ' PURPOSE: Exit the demo, being sure to stop first.                      '
     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
     Private Sub ExitBtn_Click(ByVal eventSender As System.Object, _
                               ByVal eventArgs As System.EventArgs) _
                               Handles ExitBtn.Click

        ' Be sure to stop before exiting

        If (DemoState = DEMO_RUNNING) Then
           Call StopBtn_Click(StopBtn, New System.EventArgs())
        End If


        ' Do exit

        Me.Close()

     End Sub

End Class
