Option Strict Off
Option Explicit On
Imports VB = Microsoft.VisualBasic
Imports SRI = System.Runtime.InteropServices


Module AcquireMod

    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    ' FILE: AcquireMod.vb           Copyright Symmetric Research 2003-2006   '
    '                                                                        '
    ' This file contains constants, global variables and general routines    '
    ' for the Visual Basic PARxCH test program.                              '
    '                                                                        '
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''



#Region " Constants "
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    '                                                                        '
    '                             CONSTANTS                                  '
    '                                                                        '
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

    Public Const DEMO_STOPPED As Integer = 0
    Public Const DEMO_RUNNING As Integer = 1

#End Region


#Region " Global Variables "
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    '                                                                        '
    '                        GLOBAL VARIABLES                                '
    '                                                                        '
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

    Public ParXchHandle As Integer
    Public DriverName As String
    Public ModelName As String
    Public XchModel As Integer
    Public PortMode As Integer
    Public ParXchAnalogChannels As Integer
    Public FifoSize As Integer
    Public Nvalues As Integer
    Public FileNum As Integer
    Public DemoState As Integer
    Public DataArray() As Integer
    Public WorkingDir As String
    Public MsgWords As String
    Public MsgResponse As VB.MsgBoxResult

    Public Const INIMAXTITLE As Integer = 12
    Public Const HDR_TOTAL_CHANNELS As Integer = 11

    <SRI.StructLayout(SRI.LayoutKind.Sequential)> _
    Public Structure Channel_Data
        Dim Channel As Integer
        <VBFixedArray(INIMAXTITLE-1)> _
        Dim Title() As Byte
    End Structure

    <SRI.StructLayout(SRI.LayoutKind.Sequential)> _
    Public Structure SrOutHdr
        <VBFixedArray(32-1)> _
        Dim Id() As Byte
        Dim Channels As Integer
        Dim RecordPts As Integer
        Dim WordSize As Integer
        Dim DataCoding As Integer
        Dim SamplingRate As Double
        Dim Gain As Double
        Dim FilterCoeff As Double
        Dim FilterScale As Double
        <VBFixedArray(HDR_TOTAL_CHANNELS-1)> _
        Dim ChannelList() As Channel_Data
    End Structure

    Public HDR As SrOutHdr

#End Region


#Region " Global Routines "

    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    '                                                                        '
    '                        GLOBAL PROCEDURES                               '
    '                                                                        '
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''


    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    ' ROUTINE: Channel_Data_Initialize                                       '
    ' PURPOSE: Initialize Channel_Data structure for fixed array size.       '
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    Public Sub Channel_Data_Initialize(ByRef Channel_Data As Channel_Data)

        ReDim Channel_Data.Title(INIMAXTITLE - 1)

    End Sub

    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    ' ROUTINE: SrOutHdr_Initialize                                           '
    ' PURPOSE: Initialize SrOutHdr structure for fixed array sizes.          '
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    Public Sub SrOutHdr_Initialize(ByRef SrOutHdr As SrOutHdr)

        Dim I As Short

        ReDim SrOutHdr.Id(32 - 1)
        ReDim SrOutHdr.ChannelList(HDR_TOTAL_CHANNELS - 1)

        For I = 0 To HDR_TOTAL_CHANNELS - 1
            Call Channel_Data_Initialize(SrOutHdr.ChannelList(I))
        Next

    End Sub

    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    ' ROUTINE: Write_Header                                                  '
    ' PURPOSE: This function writes the output file header info.             '
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    Public Sub Write_Header(ByRef Nchan As Integer, _
                            ByRef Npts As Integer, _
                            ByRef DataFormat As Integer, _
                            ByRef SampleRate As Double, _
                            ByRef GainLog As Integer)

       Dim I As Short
       Dim FileId As Short
       Dim FileName As String



       ' Set up header

       Call SrOutHdr_Initialize(HDR)
       
       HDR.Id(0) = &H53S ' S
       HDR.Id(1) = &H52S ' R
       HDR.Id(2) = &H20S '
       HDR.Id(3) = &H50S ' P
       HDR.Id(4) = &H41S ' A
       HDR.Id(5) = &H52S ' R
       HDR.Id(6) = &H78S ' x
       HDR.Id(7) = &H43S ' C
       HDR.Id(8) = &H48S ' H


    #If GPS_AVAILABLE Then
       HDR.Id(9) = &H20S  '
       HDR.Id(10) = &H77S ' w
       HDR.Id(11) = &H69S ' i
       HDR.Id(12) = &H74S ' t
       HDR.Id(13) = &H68S ' h
       HDR.Id(14) = &H20S '
       HDR.Id(15) = &H47S ' G
       HDR.Id(16) = &H50S ' P
       HDR.Id(17) = &H53S ' S
       HDR.Id(18) = &H0S  ' NULL
    #Else
       HDR.Id(9)  = &H0s   ' NULL
    #End If


       HDR.Channels = Nchan
       HDR.RecordPts = Npts
       HDR.WordSize = 4
       If (DataFormat = PARXCH_DF_OFFSET) Then
          HDR.DataCoding = 0
       Else 'PARXCH_DF_SIGNED
          HDR.DataCoding = 1
       End If

       HDR.SamplingRate = SampleRate
       HDR.Gain = 1 ^ GainLog
       HDR.FilterCoeff = 0.0#
       HDR.FilterScale = 1.0#

       For I = 0 To HDR.Channels - 1 Step 1
          HDR.ChannelList(I).Channel = I
          HDR.ChannelList(I).Title(0) = &H43S ' C
          HDR.ChannelList(I).Title(1) = &H48S ' H
          HDR.ChannelList(I).Title(2) = &H30S ' 0
          HDR.ChannelList(I).Title(3) = &H30S + I
          HDR.ChannelList(I).Title(4) = &H0S
       Next I


       ' Set file name and id

       FileName = WorkingDir & "\" & "DEMO.HDR"
       FileId = FreeFile



       ' Open file

       FileOpen(FileId, FileName, OpenMode.Binary, OpenAccess.Write, OpenShare.Shared)

       If (Err.Number <> 0) Then
          MsgWords = "ERROR " & Err.Number & ": " & Err.Description
          MsgWords = MsgWords & vbCrLf & "Can't open file " & FileName
          MsgResponse = MsgBox(MsgWords, MsgBoxStyle.OKOnly, "ERR_FILE")
          Err.Clear()
       End If



       ' Output data

       FilePut(FileId, HDR, 1)

       If (Err.Number <> 0) Then
          MsgWords = "ERROR " & Err.Number & ": " & Err.Description
          MsgWords = MsgWords & vbCrLf & "Can't write to file " & FileName
          MsgResponse = MsgBox(MsgWords, MsgBoxStyle.OKOnly, "ERR_FILE")
          Err.Clear()
       End If



       ' Close file

       FileClose(FileId)

    End Sub

    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    ' ROUTINE: Write_Data                                                    '
    ' PURPOSE: This function writes the output file data array.              '
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    Public Sub Write_Data()

       Dim FileId As Short
       Dim FileName As String


       ' Set file name (using zero padded decimal digits) and id

       FileName = WorkingDir & "\" & New String("0", 8 - Len(Trim(Str(FileNum)))) _
                 & Trim(Str(FileNum)) & ".OUT"
       FileId = FreeFile()



       ' Open file

       FileOpen(FileId, FileName, OpenMode.Binary, OpenAccess.Write, OpenShare.Shared)

       If (Err.Number <> 0) Then
          MsgWords = "ERROR " & Err.Number & ": " & Err.Description
          MsgWords = MsgWords & vbCrLf & "Can't open file " & FileName
          MsgResponse = MsgBox(MsgWords, MsgBoxStyle.OkOnly, "ERR_FILE")
          Err.Clear()
       End If



       ' Output data

       FilePut(FileId, DataArray, 1)

       If (Err.Number <> 0) Then
          MsgWords = "ERROR " & Err.Number & ": " & Err.Description
          MsgWords = MsgWords & vbCrLf & "Can't write to file " & FileName
          MsgResponse = MsgBox(MsgWords, MsgBoxStyle.OkOnly, "ERR_FILE")
          Err.Clear()
       End If



       ' Close file and increment number

       FileClose(FileId)
       FileNum = FileNum + 1

    End Sub

#End Region

End Module
